@@ -156,8 +156,10 @@ fn consistent_access_fs_rw() {
         let access_all = AccessFs::from_all(abi);
         let access_read = AccessFs::from_read(abi);
         let access_write = AccessFs::from_write(abi);
+        let access_file = AccessFs::from_file(abi);
         assert_eq!(access_read, !access_write & access_all);
         assert_eq!(access_read | access_write, access_all);
+        assert_eq!(access_file, access_all & ACCESS_FILE);
     }
 }
 
@@ -224,7 +226,7 @@ where
 ///     Ok(PathBeneath::new(PathFd::new("/home")?, AccessFs::ReadDir))
 /// }
 /// ```
-#[cfg_attr(test, derive(Debug))]
+#[derive(Debug)]
 pub struct PathBeneath<F> {
     attr: uapi::landlock_path_beneath_attr,
     // Ties the lifetime of a file descriptor to this object.
@@ -500,7 +502,7 @@ fn path_beneath_check_consistency() {
 ///     Ok(PathBeneath::new(fd, access))
 /// }
 /// ```
-#[cfg_attr(test, derive(Debug))]
+#[derive(Debug)]
 pub struct PathFd {
     fd: OwnedFd,
 }
@@ -550,9 +552,16 @@ fn path_fd() {
 ///
 /// # Note
 ///
-/// Landlock rules operate on file descriptors, not paths. This is a helper to create rules based on paths. **It
-/// silently ignores paths that cannot be opened**, and automatically adjusts access rights according to file types when
-/// possible.
+/// From the kernel's perspective, Landlock rules operate on file descriptors, not paths.
+/// This is a helper to create rules based on paths. Here, `path_beneath_rules()` silently ignores
+/// paths that cannot be opened, hence making the obtainment of a file descriptor impossible. When
+/// possible and for a given path, `path_beneath_rules()` automatically adjusts [access rights](`AccessFs`),
+/// depending on whether a directory or a file is present at that said path.
+///
+/// This behavior is the result of [`CompatLevel::BestEffort`], which is the default compatibility level of
+/// all created rulesets. Thus, it applies to the example below. However, if [`CompatLevel::HardRequirement`]
+/// is set using [`Compatible::set_compatibility`], attempting to create an incompatible rule at runtime will cause
+/// this crate to raise an error instead.
 ///
 /// # Example
 ///
